require 'fluent/plugin/output'
require 'reckoner_cdp'

Encoding.default_internal = 'utf-8'

module Fluent
  module Plugin
    class ReckonerCdpOutput < Fluent::Plugin::Output
      Fluent::Plugin.register_output("reckoner_cdp", self)

      config_param :workflow_id, :integer
      config_param :token, :string
      config_param :endpoint, :string, default: Reckoner::Cdp::STREAMING_ENDPOINT
      config_param :client_id, :string, default: Reckoner::Cdp::STREAMING_CLIENT_ID

      MAXIMUM_QUERY_STRING_LENGTH = 10000

      def configure(conf)
        super
        @token = conf['token']
        @endpoint = conf['endpoint']
        @client_id = conf['client_id']
      end

      def start
        super
        @client = Reckoner::Cdp::Client.new(token, endpoint: @endpoint, client_id: @client_id)
      end

      def write(chunk)
        data = []
        chunk.each do |_, record|
          data << record
          if data.join.length >= MAXIMUM_QUERY_STRING_LENGTH
            data.delete_at(-1)
            @client.streaming_insert(@workflow_id, data)
            data = [record]
          end
        end
        @client.streaming_insert(@workflow_id, data)
      end
    end
  end
end
